/**************************************************************************//**
*   @file   CN0150.c
*   @brief  Implementation of CN0150 Driver for Microblaze processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define CN0150_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "cn0150.h"
#include "../drivers/spi.h"
#include "../drivers/gpio.h"

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
static unsigned char 	counter			= 1;	// Current position in the array.
static unsigned int 	averageValue	= 0;	// Average value.
static unsigned char 	cnt				= 0;	// Counter used for calculation of the average value.

/*****************************************************************************/
/*  Aliases for the uC Probe data                                            */
/*****************************************************************************/
volatile unsigned char* readAdcButton	= &CN0150_data[0].uchData;	// Position of the Read ADC button.
volatile unsigned char* sampleSizeSlider= &CN0150_data[65].uchData;	// Position of the Sample Size slider.

/**************************************************************************//**
 * @brief Receives 16 bits from SPI and transmits 0x2100.
 * 		  0x2100 - the on-chip reference is disabled; mode 2.
 *
 * @param base - Base address.
 *
 * @return rxData - Received data.
******************************************************************************/
unsigned int Read16BitsSpi(unsigned int base)
{
	unsigned char rxBuf[2];
	unsigned char txBuf[2] = {0x21, 0x00};
	unsigned int rxData;
	
	SPI_TransferData(CN0150_SPI, 2, (char*)txBuf, 2, (char*)rxBuf, 1);
	rxData = ((unsigned int)rxBuf[0] << 8) + rxBuf[1];

	return(rxData);
}

/**************************************************************************//**
 * @brief Initializes the communication with the device.
 * 		  Initializes variables.
 *
 * @param None.
 *
 * @return TRUE.
******************************************************************************/
int CN0150_Init(void)
{
	SPI_Init(CN0150_SPI, 0, 1, 1);
	
	*sampleSizeSlider = 100;
	counter = 1;

	return(TRUE);
}

/**************************************************************************//**
 * @brief Updates the device's monitored and controlled parameters
 *
 * @param : None.
 *
 * @return: TRUE.
******************************************************************************/
int CN0150_UpdateParameters(void)
{
	if(*readAdcButton)		// If the Read ADC button was pressed, reads and stores data from ADC.
	{
		averageValue = 0;
		for(cnt = 0; cnt < *sampleSizeSlider; cnt ++)
		{
			averageValue += Read16BitsSpi(CN0150_SPI);
		}
		averageValue = (averageValue / *sampleSizeSlider);
		CN0150_data[counter].uiData = (averageValue & 0x0FFF);
		if(counter == 64)
		{
			counter = 1;
		}
		else
		{
			counter ++;
		}
		*readAdcButton = 0;
	}

	return(TRUE);
}
